import _extends from "@babel/runtime/helpers/extends";

/*
 * @Description: markdown 转化
 */
import { getTransformValue } from './transform';
import remark from 'remark';
export * from './interface';

/** 转换 代码*/
var getProcessor = scope => {
  try {
    var child = remark.parse(scope);
    return child.children;
  } catch (err) {
    console.warn(err);
  }
};
/**
 * ```js
 * 'mdx:preview' => ''  // Empty
 * 'mdx:preview:demo12' => 'demo12' // return meta id => 'demo12'
 * ```
 * @param meta string
 * @returns string?
 */


export var getMetaId = function getMetaId(meta) {
  if (meta === void 0) {
    meta = '';
  }

  var [metaRaw = ''] = /mdx:(.[\w|:]+)/i.exec(meta) || [];
  return metaRaw.replace(/^mdx:preview:?/, '');
};
/** 获取需要渲染的代码块 **/

var getCodeBlock = function getCodeBlock(child, opts) {
  if (opts === void 0) {
    opts = {};
  }

  var {
    lang = ['jsx', 'tsx']
  } = opts; // 获取渲染部分

  var codeBlock = {};

  try {
    child.forEach(item => {
      if (item && item.type === 'code' && lang.includes(item.lang)) {
        var line = item.position.start.line;
        var metaId = getMetaId(item.meta);

        if (metaId) {
          var name = typeof metaId === 'string' ? metaId : line;
          var funName = "BaseCode" + line;
          var returnCode = getTransformValue(item.value, funName + "." + lang, funName, opts);
          codeBlock[line] = {
            code: returnCode,
            name,
            language: item.lang,
            value: item.value
          };
        }
      }
    });
  } catch (err) {
    console.warn(err);
  }

  return codeBlock;
};

var createStr = codeBlock => {
  var baseCodeStr = "";
  var baseCodeObjStr = "";
  var codeBlockValue = "";
  var languageStr = "";

  try {
    Object.entries(codeBlock).forEach(_ref => {
      var [key, item] = _ref;
      var {
        code,
        value,
        language,
        name
      } = item;
      baseCodeStr += code + ";\n";
      baseCodeObjStr += name + ":BaseCode" + key + ",\n";
      codeBlockValue += name + ":" + JSON.stringify(value) + ",\n";
      languageStr += name + ":`" + language + "`,\n";
    });
  } catch (err) {
    console.warn(err);
  }

  var indexStr = baseCodeStr + " const languages={" + languageStr + "};\n const codeBlock={" + codeBlockValue + "};\n const components={" + baseCodeObjStr + "}";
  return indexStr;
};

export var getCodeBlockString = function getCodeBlockString(scope, opts) {
  if (opts === void 0) {
    opts = {};
  }

  var children = getProcessor(scope);
  var codeBlock = getCodeBlock(children, opts);
  var result = createStr(codeBlock);
  return result;
};
/**
 * `mdCodeModulesLoader` method for adding `markdown-react-code-preview-loader` to webpack config.
 * @param {webpack.Configuration} config webpack config
 * @param {string[]} lang Parsing language
 * @param {Options} option Loader Options
 * @returns {webpack.Configuration}
 */

export var mdCodeModulesLoader = function mdCodeModulesLoader(config, lang, option) {
  if (option === void 0) {
    option = {};
  }

  config.module.rules.forEach(ruleItem => {
    if (typeof ruleItem === 'object') {
      if (ruleItem.oneOf) {
        ruleItem.oneOf.unshift({
          test: /.md$/,
          use: [{
            loader: 'markdown-react-code-preview-loader',
            options: _extends({
              lang
            }, option)
          }]
        });
      }
    }
  });
  return config;
};
//# sourceMappingURL=index.js.map