"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault")["default"];

Object.defineProperty(exports, "__esModule", {
  value: true
});
var _exportNames = {
  getMetaId: true,
  getCodeBlockString: true,
  mdCodeModulesLoader: true
};
exports.mdCodeModulesLoader = exports.getMetaId = exports.getCodeBlockString = void 0;

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread2"));

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _transform = require("./transform");

var _remark = _interopRequireDefault(require("remark"));

var _interface = require("./interface");

Object.keys(_interface).forEach(function (key) {
  if (key === "default" || key === "__esModule") return;
  if (Object.prototype.hasOwnProperty.call(_exportNames, key)) return;
  if (key in exports && exports[key] === _interface[key]) return;
  Object.defineProperty(exports, key, {
    enumerable: true,
    get: function get() {
      return _interface[key];
    }
  });
});

/*
 * @Description: markdown 转化
 */

/** 转换 代码*/
var getProcessor = function getProcessor(scope) {
  try {
    var child = _remark["default"].parse(scope);

    return child.children;
  } catch (err) {
    console.warn(err);
  }
};
/**
 * ```js
 * 'mdx:preview' => ''  // Empty
 * 'mdx:preview:demo12' => 'demo12' // return meta id => 'demo12'
 * ```
 * @param meta string
 * @returns string?
 */


var getMetaId = function getMetaId() {
  var meta = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';

  var _ref = /mdx:(.[\w|:]+)/i.exec(meta) || [],
      _ref2 = (0, _slicedToArray2["default"])(_ref, 1),
      _ref2$ = _ref2[0],
      metaRaw = _ref2$ === void 0 ? '' : _ref2$;

  return metaRaw.replace(/^mdx:preview:?/, '');
};
/** 获取需要渲染的代码块 **/


exports.getMetaId = getMetaId;

var getCodeBlock = function getCodeBlock(child) {
  var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var _opts$lang = opts.lang,
      lang = _opts$lang === void 0 ? ['jsx', 'tsx'] : _opts$lang; // 获取渲染部分

  var codeBlock = {};

  try {
    child.forEach(function (item) {
      if (item && item.type === 'code' && lang.includes(item.lang)) {
        var line = item.position.start.line;
        var metaId = getMetaId(item.meta);

        if (metaId) {
          var name = typeof metaId === 'string' ? metaId : line;
          var funName = "BaseCode".concat(line);
          var returnCode = (0, _transform.getTransformValue)(item.value, "".concat(funName, ".").concat(lang), funName, opts);
          codeBlock[line] = {
            code: returnCode,
            name: name,
            language: item.lang,
            value: item.value
          };
        }
      }
    });
  } catch (err) {
    console.warn(err);
  }

  return codeBlock;
};

var createStr = function createStr(codeBlock) {
  var baseCodeStr = "";
  var baseCodeObjStr = "";
  var codeBlockValue = "";
  var languageStr = "";

  try {
    Object.entries(codeBlock).forEach(function (_ref3) {
      var _ref4 = (0, _slicedToArray2["default"])(_ref3, 2),
          key = _ref4[0],
          item = _ref4[1];

      var code = item.code,
          value = item.value,
          language = item.language,
          name = item.name;
      baseCodeStr += "".concat(code, ";\n");
      baseCodeObjStr += "".concat(name, ":BaseCode").concat(key, ",\n");
      codeBlockValue += "".concat(name, ":").concat(JSON.stringify(value), ",\n");
      languageStr += "".concat(name, ":`").concat(language, "`,\n");
    });
  } catch (err) {
    console.warn(err);
  }

  var indexStr = "".concat(baseCodeStr, " const languages={").concat(languageStr, "};\n const codeBlock={").concat(codeBlockValue, "};\n const components={").concat(baseCodeObjStr, "}");
  return indexStr;
};

var getCodeBlockString = function getCodeBlockString(scope) {
  var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var children = getProcessor(scope);
  var codeBlock = getCodeBlock(children, opts);
  var result = createStr(codeBlock);
  return result;
};
/**
 * `mdCodeModulesLoader` method for adding `markdown-react-code-preview-loader` to webpack config.
 * @param {webpack.Configuration} config webpack config
 * @param {string[]} lang Parsing language
 * @param {Options} option Loader Options
 * @returns {webpack.Configuration}
 */


exports.getCodeBlockString = getCodeBlockString;

var mdCodeModulesLoader = function mdCodeModulesLoader(config, lang) {
  var option = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  config.module.rules.forEach(function (ruleItem) {
    if ((0, _typeof2["default"])(ruleItem) === 'object') {
      if (ruleItem.oneOf) {
        ruleItem.oneOf.unshift({
          test: /.md$/,
          use: [{
            loader: 'markdown-react-code-preview-loader',
            options: (0, _objectSpread2["default"])({
              lang: lang
            }, option)
          }]
        });
      }
    }
  });
  return config;
};

exports.mdCodeModulesLoader = mdCodeModulesLoader;
//# sourceMappingURL=index.js.map